/// @PACKAGEDEFAULT MissionFrameworkACV

// Used to know if the Mission Framework is present or not.
TYR_MISSION_FRAMEWORK = true;

TYR_FNCT_ExecuteMissionCommandServer = {
	diag_log Format["(%1)[%2]: %3: %4",diag_tickTime,"TYR_FNCT_mission.sqf","TYR_FNCT_ExecuteMissionCommand",_this];
	["d_s_mexecute", _this] call XNetCallEvent;
};

TYR_FNCT_ExecuteMissionCommandAll = {
	diag_log Format["(%1)[%2]: %3: %4",diag_tickTime,"TYR_FNCT_mission.sqf","TYR_FNCT_ExecuteMissionCommand",_this];
	["d_all_mexecute", _this] call XNetCallEvent;
};

TYR_FNCT_NextJipID = {
	private "_id";
	// _id = ["TYR_VAR_JIP_NEXTID",0] call XNetGetJIP;
	// if (_id == 0) then { _id = 1;};
	// ["TYR_VAR_JIP_NEXTID",(_id + 1)] call XNetSetJIP; 
	_id = call XfMissionNextJipID;
	_id;
};

TYR_FNCT_RanCircle = {
	private ["_center", "_radius", "_center_x", "_center_y", "_ret_val", "_co", "_angle", "_x1", "_y1"];
	_center = _this select 0;_radius = _this select 1;
	_center_x = _center select 0;_center_y = _center select 1;
	_ret_val = [];_co = 0;
	while {count _ret_val == 0 && _co < 100} do {
		_angle = floor (random 360);
		_x1 = _center_x - ((random _radius) * sin _angle);
		_y1 = _center_y - ((random _radius) * cos _angle);
		if (!(surfaceiswater [_x1, _y1])) then {_ret_val = [_x1, _y1, 0]};
		if (count _ret_val == 0) then {_co = _co + 1};
	};
	if (count _ret_val == 0) then { _ret_val = _center};
	_ret_val
};

/// @FUNCTION
/// @NAME TYR_FNCT_CleanMission
/// @DESC 
/// @ACCESS Public
/// @MP True
/// @RETURN Nothing
/// @REQUIRE MissionFramework
/// @REMARK 
/// @EXAMPLE [] call TYR_FNCT_CleanMission;
TYR_FNCT_CleanMission = {
	["d_all_mclean",["ALL",[True]]] call XNetCallEvent;
};


/// @FUNCTION
/// @NAME TYR_FNCT_GetVar
/// @DESC 
/// @ACCESS Public
/// @MP True
/// @PARAM String The name of the variable to retrieve
/// @PARAM Variant The default value we want back in case the variable doesn't exist
/// @RETURN Variant The content of the variable
/// @REQUIRE MissionFramework
/// @REMARK 
/// @EXAMPLE ["JIPM_DESTROY_1"] call TYR_FNCT_GetVar;
TYR_FNCT_GetVar = {
	private ["_name","_default"];
	_default = false;
	if (typeName _this == "STRING") then {
		_name = _this;
		_default = [_name,_default] call XNetGetJIP;
	} else {
		_name = _this select 0;
		if (count _this > 1) then {_default = _this select 1;};
		_default = [_name,_default] call XNetGetJIP;
	};
	_default;
};



/// @FUNCTION
/// @NAME TYR_FNCT_SetEnvironment
/// @DESC Sets the environment of the game to new values. You can change the weather and time of the game. This will synchronize on all computers.
/// @ACCESS Public
/// @MP True
/// @PARAM Scalar The hour we want, if -1 it is ignored.
/// @PARAM Scalar The rain intensity we want, if -1 it is ignored. 0 is minimum rain and 1 is maximum.
/// @PARAM Scalar The overcast we want, if -1 it is ignored. 0 is minimum overcast and 1 is maximum.
/// @PARAM Scalar The fog we want, if -1 it is ignored. 0 is minimum fog and 1 is maximum.
/// @RETURN Nothing
/// @REQUIRE MissionFramework
/// @REMARK 
/// @EXAMPLE [-1,0.2,0.3,0] call TYR_FNCT_SetEnvironment;
TYR_FNCT_SetEnvironment = {
	private ["_time","_rain","_overcast","_fog","_newtime"];
	_time = -1;
	_rain = -1;
	_overcast = -1;
	_fog = -1;
	
	if (count _this > 0) then {	_time = round(_this select 0); };
	if (count _this > 1) then {	_rain = _this select 1; };
	if (count _this > 2) then {	_overcast = _this select 2; };
	if (count _this > 3) then {	_fog = _this select 3; };
	
	if (_time > 23) then { _time = 23; };
	If (_rain > 1) then { _rain = 1; };
	If (_overcast > 1) then { _overcast = 1; };
	If (_fog > 1) then { _fog = 1; };
	
	if (_time >= 0) then {
		_newtime = [_time] call XfMissionGetDate;
		_newtime call XfMissionSetDate;
	};
	If (_rain >= 0 || _overcast >= 0 || _fog >= 0) then {
		If (_rain < 0) then { _rain = rain; };
		If (_overcast < 0) then { _overcast = overcast; };
		If (_fog < 0) then { _fog = fog; };
		[60, _rain, _overcast, _fog] call XfMissionSetWeather;
	};
};



/// @FUNCTION
/// @NAME TYR_FNCT_TaskCreate
/// @DESC 
/// @ACCESS Public
/// @MP True
/// @PARAM String Task Title
/// @PARAM String The description of the task. Note you can use the <br/> tag to go to the next line. @DEFAULT $STR_TMF_TASKNODESC
/// @RETURN String The task unique name
/// @REQUIRE MissionFramework
/// @EXAMPLE ["Kill the traitor","Go to the town of Kamenyy and kill Olaf"] call TYR_FNCT_TaskCreate;
TYR_FNCT_TaskCreate = {
	private ["_title","_name","_desc","_status","_success_var"];
	_title = _this select 0;
	_desc = format[localize "STR_TMF_TASKNODESC",_title];
	if (count _this > 1) then {_desc = _this select 1;};
	_name = Format["mission_task_%1",(call TYR_FNCT_NextJipID)];
	[_name,_title, _desc,"",[],"Created"] call XfMissionTask;
	_name;
};

/// @FUNCTION
/// @NAME TYR_FNCT_TaskUpdate
/// @DESC 
/// @ACCESS Public
/// @MP True
/// @PARAM String Task Unique name
/// @PARAM String The current status of the task among: "Created","Succeeded","Failed","Canceled". @DEFAULT "Succeeded"
/// @RETURN String The task unique name
/// @REQUIRE MissionFramework
/// @EXAMPLE ["mission_task_1","Succeeded"] call TYR_FNCT_TaskUpdate;
TYR_FNCT_TaskUpdate = {
	private ["_title","_name","_desc","_status","_success_var"];
	_name = _this select 0;
	_status = "Succeeded";
	if (count _this > 1) then {_status = _this select 1;};
	_status = switch (ToUpper(_type)) do {
		case "CREATED":{"Created"};
		case "FAILED":{"Failed"};
		case "CANCELED":{"Canceled"};
		default {"Succeeded"};
	};
	[_name,"", "","",[],_status] call XfMissionTask;
	_name;
};

/// @FUNCTION
/// @NAME TYR_FNCT_ShapeMarker
/// @DESC 
/// @ACCESS Public
/// @MP True
/// @PARAM Position The position of the marker
/// @PARAM Number The size of the marker
/// @PARAM Boolean True if it's a rectangle  @DEFAULT True (rectangle)
/// @PARAM String The color of the marker. Among other values: "Default","ColorBlack","ColorRed","ColorGreen","ColorBlue","ColorYellow","ColorOrange","ColorWhite" @DEFAULT "Default"
/// @PARAM Number Alpha value of the marker. 0 = hidden, 1 = fully visible. @DEFAULT 0 (hidden)
/// @RETURN String The marker name
/// @REQUIRE MissionFramework
/// @REMARK 
/// @EXAMPLE [getPos Player,200,True,"ColorRed",1] call TYR_FNCT_ShapeMarker;
TYR_FNCT_ShapeMarker = {
	private ["_position","_size","_shape","_icon","_color","_alpha"];
	_position = _this select 0;
	_size = _this select 1;
	_shape = "RECTANGLE";
	if (count _this > 2) then {_icon = if (_this select 2) then {"RECTANGLE"} else {"ELLIPSE"};};
	_color = "Default";
	if (count _this > 3) then {_color = _this select 3;};
	_alpha = 0;
	if (count _this > 4) then {_alpha = _this select 4;};
	if (_alpha > 1) then { _alpha = 1; };
	if (_alpha < 0) then { _alpha = 0; };
	if (TypeName _size != "ARRAY") then {_size = [_size, _size];};
	_name = Format["marker_shape_%1",(call TYR_FNCT_NextJipID)];
	[false,[_name,_position, _shape, _color, _size,"",0,"","FDiagonal",_alpha]] call XfMissionMarker;
	_name;
};
	
/// @FUNCTION
/// @NAME TYR_FNCT_IconMarker
/// @DESC 
/// @ACCESS Public
/// @MP True
/// @PARAM Position The position of the marker
/// @PARAM String The caption of the marker
/// @PARAM String The icon of the marker. Among other values: "hd_objective","hd_flag","hd_arrow","hd_ambush","hd_destroy","hd_start","hd_end","hd_pickup","hd_join","hd_warning","hd_unknown","hd_dot" @DEFAULT "hd_objective"
/// @PARAM String The icon color of the marker. Among other values: "Default","ColorBlack","ColorRed","ColorGreen","ColorBlue","ColorYellow","ColorOrange","ColorWhite" @DEFAULT "Default"
/// @PARAM Number Alpha value of the icon. 0 = hidden, 1 = fully visible. @DEFAULT 0 (hidden)
/// @RETURN String The marker name
/// @REQUIRE MissionFramework
/// @REMARK 
/// @EXAMPLE [getPos Player,"Target Here","hd_objective","Default",1] call TYR_FNCT_IconMarker;
TYR_FNCT_IconMarker = {
	private ["_position","_name","_caption","_icon","_color","_alpha"];
	_position = _this select 0;
	_caption = _this select 1;
	_icon = "hd_objective";
	if (count _this > 2) then {_icon = _this select 2;};
	_color = "Default";
	if (count _this > 3) then {_color = _this select 3;};
	_alpha = 0;
	if (count _this > 4) then {_alpha = _this select 4;};
	if (_alpha > 1) then { _alpha = 1; };
	if (_alpha < 0) then { _alpha = 0; };
	_name = Format["marker_icon_%1",(call TYR_FNCT_NextJipID)];
	[false,[_name,_position, "ICON", _color, [0.7,0.7],_caption,0,_icon,"",_alpha]] call XfMissionMarker;
	_name;
};


/// @FUNCTION
/// @NAME TYR_FNCT_Mission_Populate
/// @DESC Fills a zone with enemies, depending on the settings it can add snipers and/or airsupport
/// @ACCESS Public
/// @MP True
/// @PARAM String The side of the units to spawn (EAST, WEST, etc)
/// @PARAM Position Position of the location to fill, it can be given with a radius. In such a case add the radius to the position like this: [[0,0,0],0]. Default radius is 200.
/// @PARAM Scalar The power of the enemy forces, a value ranging from 1 to 7. @DEFAULT 0 = Automatic (based on player number)
/// @PARAM Scalar The amount of sniper units on the zone (if appropriate buildings are present), a value ranging from 0 to 8. The result is always random between half the value and max value. @DEFAULT 1 = 1 possible sniper.
/// @PARAM Scalar The power of the air support on the zone, a value randing from 0 to 4. @DEFAULT 1 = 1 aerial unit.
/// @PARAM String The displayed message when the zone is conquered. @DEFAULT "" No message.
/// @RETURN Array An array containing the variable which indicates that the zone has been conquered. [conquered_varname]
/// @REQUIRE MissionFramework
/// @REMARK The created units are accessible through the ACV.
/// @EXAMPLE _res = ["EAST",ACV_PosClick] call TYR_FNCT_Mission_Populate; // All defaults
/// @EXAMPLE _res = ["EAST",ACV_PosClick,0,4] call TYR_FNCT_Mission_Populate; // Default power, Default AirSupport and 4 possible snipers in place of default.
TYR_FNCT_Mission_Populate = {
	private ["_side","_position"];
	_side = toUpper(_this select 0);
	_position = _this select 1;
	_power = 0;
	_coeff = 0;
	_snipers = 1;
	_airsupport = 1;
	_jip_var = Format["JIPM_CONQUER_%1", (call TYR_FNCT_NextJipID)];
	_conquer_pos = _position;
	_conquer_rad = 320; // for conquered zone we take it a bit larger for wandering units.
	_conquer_msg = "";
	If (typename (_conquer_pos select 0) == "ARRAY") then {
		_conquer_rad = _conquer_pos select 1;
		_conquer_pos = _conquer_pos select 0;
		_conquer_rad = _conquer_rad * 1.7; // for conquered zone we take it a bit larger for wandering units.
	};
	if (count _this > 2) then {
		if (typeName (_this select 2) == "SCALAR") then {_power = _this select 2;};
		if (_power < 0) then { _power = 0; };
		if (_power > 7) then { _power = 7; };
		if (_power > 5) then { _coeff = _power - 5; };
	};
	if (count _this > 3) then {
		if (typeName (_this select 3) == "SCALAR") then {_snipers = _this select 3;};
		if (_snipers < 0) then { _snipers = 0; };
		if (_snipers > 8) then { _snipers = 8; };
	};
	if (count _this > 4) then {
		if (typeName (_this select 4) == "SCALAR") then {_airsupport = _this select 4;};
		if (_airsupport < 0) then { _airsupport = 0; };
		if (_airsupport > 4) then { _airsupport = 4; };
	};
	if (count _this > 5) then {
		if (typeName (_this select 5) == "STRING") then {_conquer_msg = _this select 5;};
	};
	_use_upsmon = true;
	if (_side == d_own_side) then {_use_upsmon = false;};
	
	["XfMissionPopulateLocation",[_side,_position,_use_upsmon,_power,_coeff]] call TYR_FNCT_ExecuteMissionCommandServer;
	["XfMissionCreateSnipersArea",[_side,_position,[floor(_snipers / 2),_snipers]]] call TYR_FNCT_ExecuteMissionCommandServer;
	// We pass the spawn position in order to directly spawn the airsupport on place. Normally we don't need to do it another way.
	[_side,_position,_airsupport,_conquer_pos] call XfMissionCallAirSupport;
	[_jip_var,false] call XNetSetJIP;
	["XfMissionCreateMissionObject",["Land_HelipadEmpty_F",_conquer_pos,"CONQUER",[_conquer_msg,[_conquer_pos, [_conquer_rad,_conquer_rad], 0, false],_jip_var]]] call TYR_FNCT_ExecuteMissionCommandServer;

	[_jip_var];
};

/// @FUNCTION
/// @NAME TYR_FNCT_Mission_Object
/// @DESC 
/// @ACCESS Public
/// @MP True
/// @PARAM Class The class of the object to create
/// @PARAM Position Position of the object, it can be given with a direction. In such a case add the direction to the position like this: [[0,0,0],0]
/// @PARAM String The object name displayed to the player
/// @PARAM String Init Line code of the object @DEFAULT Empty
/// @RETURN Nothing
/// @REQUIRE MissionFramework
/// @REMARK The object itself is accessible through the ACV.
/// @EXAMPLE ["Land_telek1",ACV_PosClick] call TYR_FNCT_Mission_Object;
TYR_FNCT_Mission_Object = {
	private ["_class","_position","_init_line"];
	_class = _this select 0;
	_position = _this select 1;
	_init_line = "";
	if (count _this > 8) then { _init_line = _this select 8;};
	["XfMissionCreateMissionObject",[_class,_position,"REGISTER",[],_init_line]] call TYR_FNCT_ExecuteMissionCommandServer;
};

/// @FUNCTION
/// @NAME TYR_FNCT_Mission_Destroy
/// @DESC 
/// @ACCESS Public
/// @MP True
/// @PARAM Class The class of the object to create
/// @PARAM Position Position of the object, it can be given with a direction. In such a case add the direction to the position like this: [[0,0,0],0]
/// @PARAM String The object name displayed to the player
/// @PARAM String Success message when the objective is destroyed @DEFAULT $STR_TMF_TARGET_DESTROYED
/// @PARAM String The name of the task to appear in the briefing. This will automatically create a task and task waypoint. @DEFAULT Empty. Do not create a task
/// @PARAM String The description of the task in the briefing. @DEFAULT Empty. Do not add a task description
/// @PARAM Number The amount of Ammo necessary to destroy the objective. @DEFAULT 1
/// @PARAM Array An array of ammo classes which are taken into account for the damages @DEFAULT ["PipeBomb"]
/// @PARAM String Init Line code of the object @DEFAULT Empty
/// @PARAM String A custom code string which will be executed when the target is destroyed. I recommend against using this. @DEFAULT Empty
/// @RETURN Array An array containing only the variable which indicates the success of the objective and the task unique name. [SuccessVariable,TaskUniqueName]
/// @REQUIRE MissionFramework
/// @REMARK The object itself is accessible through the ACV.
/// @EXAMPLE ["Land_telek1",ACV_PosClick,"Radio Tower","You destroyed the tower","","",1,["PipeBomb"]] call TYR_FNCT_Mission_Destroy;
TYR_FNCT_Mission_Destroy = {
	//["Land_telek1",[[6076.28,10479.3,0],355],"DESTROY",["The Tower is destroyed!","jip_tower_is_destroyed",True,1,["PipeBomb"]]] call XfMissionCreateMissionObject;
	private ["_class","_position","_object_name","_success_msg","_taskname","_taskdesc","_jip_var","_amount_ammo","_ammo_types","_init_line","_custom_code","_taskunique","_wp_pos"];
	_class = _this select 0;
	_position = _this select 1;
	_jip_var = Format["JIPM_DESTROY_%1", (call TYR_FNCT_NextJipID)];
	_object_name = localize "STR_TMF_TARGET_DESTROY_OBJECT";
	if (count _this > 2) then { _object_name = _this select 2;};
	_success_msg = localize "STR_TMF_TARGET_DESTROYED";
	if (count _this > 3) then { _success_msg = _this select 3;};
	_taskname = "";
	if (count _this > 4) then { _taskname = _this select 4;};
	_taskdesc = "";
	if (count _this > 5) then { _taskdesc = _this select 5;};
	_amount_ammo = 1;
	if (count _this > 6) then { _amount_ammo = _this select 6;};
	_ammo_types = ["PipeBomb"];
	if (count _this > 7) then { if (typeName (_this select 7) == "ARRAY") then {_ammo_types = _this select 7;};};
	_init_line = "";
	if (count _this > 8) then { _init_line = _this select 8;};
	_custom_code = "";
	if (count _this > 9) then { _custom_code = _this select 9;};
	_taskunique = "";
	if (_taskName != "") then {
		if (_taskdesc == "") then {_taskdesc = format[localize "STR_TMF_TASKNODESC" ,_taskname];};
		_taskunique = Format["JIPM_TASK_%1", (call TYR_FNCT_NextJipID)];
		_wp_pos = _position;
		if (typeName (_position select 0) == "ARRAY") then { _wp_pos = (_position select 0); };
		_wp_pos = [_wp_pos, 100] call TYR_FNCT_RanCircle;
		[true,[Format["marker_%1",_taskunique], _wp_pos, "ICON", "ColorBlue", [0.7,0.7],_object_name,0,"hd_objective","",1]] call XfMissionMarker;
		[_taskunique, _taskName, _taskdesc, _taskName, _wp_pos, "Created",_jip_var,""] call XfMissionTask;
	};
	["XfMissionCreateMissionObject",[_class,_position,"DESTROY",[_success_msg,_jip_var,True,_amount_ammo,_ammo_types,_custom_code],_init_line]] call TYR_FNCT_ExecuteMissionCommandServer;
	[_jip_var,_taskunique];
};

/// @FUNCTION
/// @NAME TYR_FNCT_Mission_Pickup
/// @DESC 
/// @ACCESS Public
/// @MP True
/// @PARAM Class The class of the object to create
/// @PARAM Position Position of the object, it can be given with a direction. In such a case add the direction to the position like this: [[0,0,0],0]
/// @PARAM String The object name displayed to the player
/// @PARAM String Success message when the objective is destroyed @DEFAULT $STR_TMF_TARGET_DESTROYED
/// @PARAM String The name of the task to appear in the briefing. This will automatically create a task and task waypoint. @DEFAULT Empty. Do not create a task
/// @PARAM String The description of the task in the briefing. @DEFAULT Empty. Do not add a task description
/// @PARAM String The displayed title of the Action attached to the object @DEFAULT $STR_TMF_TARGET_PICKUP_ACTION
/// @PARAM String Init Line code of the object @DEFAULT Empty
/// @PARAM String A custom code string which will be executed when the target is destroyed. I recommend against using this. @DEFAULT Empty
/// @RETURN Array An array containing only the variable which indicates the success of the objective and the task unique name. [SuccessVariable,TaskUniqueName]
/// @REQUIRE MissionFramework
/// @REMARK The object itself is accessible through the ACV.
/// @EXAMPLE ["EvMoney", ACV_PosClick, "Stolen Money", "You found the money", "Find Money","Look in the town to find the money","Pickup Money"] call TYR_FNCT_Mission_Pickup
TYR_FNCT_Mission_Pickup = {
	// ["EvMoney",[5957.36,10284,0],"PICKUP",["Pickup Money","You found the money","jip_money_found","Take the money",True]] call XfMissionCreateMissionObject;
	private ["_class","_position","_object_name","_success_msg","_taskname","_taskdesc","_jip_var","_action_title","_custom_code","_taskunique","_wp_pos","_init_line"];
	_class = _this select 0;
	_position = _this select 1;
	_jip_var = Format["JIPM_PICKUP_%1", (call TYR_FNCT_NextJipID)];
	_object_name = localize "STR_TMF_TARGET_PICKUP_OBJECT";
	if (count _this > 2) then { _object_name = _this select 2;};
	_success_msg = Format[localize "STR_TMF_TARGET_PICKEDUP",_object_name];
	if (count _this > 3) then { _success_msg = _this select 3;};
	_taskname = "";
	if (count _this > 4) then { _taskname = _this select 4;};
	_taskdesc = "";
	if (count _this > 5) then { _taskdesc = _this select 5;};
	_action_title = Format[localize "STR_TMF_TARGET_PICKUP_ACTION",_object_name];;
	if (count _this > 6) then { _action_title = _this select 6;};
	_init_line = "";
	if (count _this > 7) then { _init_line = _this select 7; };
	_custom_code = "";
	if (count _this > 8) then { _custom_code = _this select 8;};
	_taskunique = "";
	if (_taskName != "") then {
		if (_taskdesc == "") then {_taskdesc = format[localize "STR_TMF_TASKNODESC" ,_taskname];};
		_taskunique = Format["JIPM_TASK_%1", (call TYR_FNCT_NextJipID)];
		_wp_pos = _position;
		if (typeName (_position select 0) == "ARRAY") then { _wp_pos = (_position select 0); };
		_wp_pos = [_wp_pos, 100] call TYR_FNCT_RanCircle;
		[true,[Format["marker_%1",_taskunique], _wp_pos, "ICON", "ColorBlue", [0.7,0.7],_taskName,0,"hd_objective","",1]] call XfMissionMarker;
		[_taskunique, _taskName, _taskdesc, _taskName, _wp_pos, "Created",_jip_var,""] call XfMissionTask;
	};
	["XfMissionCreateMissionObject",[_class,_position,"PICKUP",[_object_name,_success_msg,_jip_var,_action_title,True,_custom_code],_init_line]] call TYR_FNCT_ExecuteMissionCommandServer;
	[_jip_var,_taskunique];
};

/// @FUNCTION
/// @NAME TYR_FNCT_Mission_Assassinate
/// @DESC 
/// @ACCESS Public
/// @MP True
/// @PARAM Class The class of the object to create
/// @PARAM Position Position of the object, it can be given with a direction. In such a case add the direction to the position like this: [[0,0,0],0]
/// @PARAM String The side of the target unit. It can be a man or empty vehicle.
/// @PARAM String Success message when the objective is destroyed @DEFAULT d_enemy_side
/// @PARAM String The name of the task to appear in the briefing. This will automatically create a task and task waypoint. @DEFAULT Empty. Do not create a task
/// @PARAM String The description of the task in the briefing. @DEFAULT Empty. Do not add a task description
/// @PARAM String Init Line code of the object @DEFAULT Empty
/// @PARAM String A custom code string which will be executed when the target is destroyed. I recommend against using this. @DEFAULT Empty
/// @RETURN Array An array containing only the variable which indicates the success of the objective and the task unique name. [SuccessVariable,TaskUniqueName]
/// @REQUIRE MissionFramework
/// @REMARK The object itself is accessible through the ACV.
/// @EXAMPLE ["RU_Profiteer2", ACV_PosClick,"EAST", "You killed the enemy general", "Kill Serge","Look in the town to find the enemy general"] call TYR_FNCT_Mission_Assassinate
TYR_FNCT_Mission_Assassinate = {
	// ["RU_Profiteer2",_grp,[5892.23,10198.2,0],"ASSASSINATE",["You killed the traitor","jip_traitor_killed"],"doStop this;"] call XfMissionCreateMissionUnit;
	private ["_class","_position","_object_name","_success_msg","_taskname","_taskdesc","_jip_var","_unit_init","_custom_code","_taskunique","_wp_pos","_init_line"];
	_class = _this select 0;
	_position = _this select 1;
	_jip_var = Format["JIPM_ASSASSINATE_%1", (call TYR_FNCT_NextJipID)];
	_side = d_enemy_side;
	if (count _this > 2) then { _side = _this select 2;};
	_success_msg = Format[localize "STR_TMF_TARGET_ASSASSINATED",_object_name];
	if (count _this > 3) then { _success_msg = _this select 3;};
	_taskname = "";
	if (count _this > 4) then { _taskname = _this select 4;};
	_taskdesc = "";
	if (count _this > 5) then { _taskdesc = _this select 5;};
	_init_line = "doStop this; ";
	if (count _this > 6) then { _init_line = _this select 6; };
	_custom_code = "";
	if (count _this > 7) then { _custom_code = _this select 7;};
	_taskunique = "";
	if (_taskName != "") then {
		if (_taskdesc == "") then {_taskdesc = format[localize "STR_TMF_TASKNODESC" ,_taskname];};
		_taskunique = Format["JIPM_TASK_%1", (call TYR_FNCT_NextJipID)];
		_wp_pos = _position;
		if (typeName (_position select 0) == "ARRAY") then { _wp_pos = (_position select 0); };
		_wp_pos = [_wp_pos, 100] call TYR_FNCT_RanCircle;
		[true,[Format["marker_%1",_taskunique], _wp_pos, "ICON", "ColorBlue", [0.7,0.7],_taskName,0,"hd_objective","",1]] call XfMissionMarker;
		[_taskunique, _taskName, _taskdesc, _taskName, _wp_pos, "Created",_jip_var,""] call XfMissionTask;
	};
	["XfMissionCreateMissionUnit",[_class,_side,_position,"ASSASSINATE",[_success_msg,_jip_var,_custom_code],_init_line]] call TYR_FNCT_ExecuteMissionCommandServer;
	[_jip_var,_taskunique];
};

/// @FUNCTION
/// @NAME TYR_FNCT_Mission_Activate
/// @DESC 
/// @ACCESS Public
/// @MP True
/// @PARAM Class The class of the object to create
/// @PARAM Position Position of the object, it can be given with a direction. In such a case add the direction to the position like this: [[0,0,0],0]
/// @PARAM String The message when the device is activated. @DEFAULT $STR_TMF_TARGET_ACTIVATED
/// @PARAM String The message when the device is deactivated. @DEFAULT $STR_TMF_TARGET_DEACTIVATED
/// @PARAM String The action text to activate the device. @DEFAULT $STR_TMF_TARGET_ACTIVATE
/// @PARAM String The action text to deactivate the device. @DEFAULT $STR_TMF_TARGET_DEACTIVATE
/// @PARAM String The name of the task to appear in the briefing. This will automatically create a task and task waypoint. @DEFAULT Empty. Do not create a task
/// @PARAM String The description of the task in the briefing. @DEFAULT Empty. Do not add a task description
/// @PARAM String Init Line code of the object @DEFAULT Empty
/// @PARAM String A custom code string which will be executed when the target is destroyed. I recommend against using this. @DEFAULT Empty
/// @RETURN Array An array containing only the variable which indicates the success of the objective and the task unique name. [SuccessVariable,TaskUniqueName]
/// @REQUIRE MissionFramework
/// @REMARK The object itself is accessible through the ACV.
/// @EXAMPLE ["PowGen_Big", ACV_PosClick, "The generator is on", "The generator is off","Activate","Deactivate"] call TYR_FNCT_Mission_Activate
TYR_FNCT_Mission_Activate = {
	// ["PowGen_Big",[5955.3,10116.7,0],"ACTIVATE",["The generator is on","The generator is off","jip_generator_active"]] call XfMissionCreateMissionObject;
	private ["_class","_position","_title_activated","_title_deactivated","_action_activate","_action_deactivate","_taskname","_taskdesc","_jip_var","_custom_code","_taskunique","_wp_pos","_init_line"];
	_class = _this select 0;
	_position = _this select 1;
	_jip_var = Format["JIPM_ACTIVATE_%1", (call TYR_FNCT_NextJipID)];
	_title_activated = localize "STR_TMF_TARGET_ACTIVATED";
	if (count _this > 2) then { _title_activated = _this select 2;};
	_title_deactivated = localize "STR_TMF_TARGET_DEACTIVATED";
	if (count _this > 3) then { _title_deactivated = _this select 3;};
	_action_activate = localize "STR_TMF_TARGET_ACTIVATE";
	if (count _this > 4) then { _action_activate = _this select 4;};
	_action_deactivate = localize "STR_TMF_TARGET_DEACTIVATE";
	if (count _this > 5) then { _action_deactivate = _this select 5;};
	_taskname = "";
	if (count _this > 6) then { _taskname = _this select 6;};
	_taskdesc = "";
	if (count _this > 7) then { _taskdesc = _this select 7;};
	_init_line = "";
	if (count _this > 8) then { _init_line = _this select 8; };
	_custom_code = "";
	if (count _this > 9) then { _custom_code = _this select 9;};
	_taskunique = "";
	if (_taskName != "") then {
		if (_taskdesc == "") then {_taskdesc = format[localize "STR_TMF_TASKNODESC" ,_taskname];};
		_taskunique = Format["JIPM_TASK_%1", (call TYR_FNCT_NextJipID)];
		_wp_pos = _position;
		if (typeName (_position select 0) == "ARRAY") then { _wp_pos = (_position select 0); };
		_wp_pos = [_wp_pos, 100] call TYR_FNCT_RanCircle;
		[true,[Format["marker_%1",_taskunique], _wp_pos, "ICON", "ColorBlue", [0.7,0.7],_taskName,0,"hd_objective","",1]] call XfMissionMarker;
		[_taskunique, _taskName, _taskdesc, _taskName, _wp_pos, "Created",_jip_var,""] call XfMissionTask;
	};
	["XfMissionCreateMissionObject",[_class,_position,"ACTIVATE",[_title_activated,_title_deactivated,_jip_var,_action_deactivate,_action_activate,False,_custom_code],_init_line]] call TYR_FNCT_ExecuteMissionCommandServer;
	[_jip_var,_taskunique];
};


/// @FUNCTION
/// @NAME TYR_FNCT_Mission_Rescue
/// @DESC Creates a unit to be freed (the unit remains in place).
/// @ACCESS Public
/// @MP True
/// @PARAM Class The class of the unit to create
/// @PARAM Position Position of the unit, it can be given with a direction. In such a case add the direction to the position like this: [[0,0,0],0]
/// @PARAM String The side of the unit to rescue. @DEFAULT Own Side
/// @PARAM String The message when the unit is at destination. @DEFAULT $STR_TMF_TARGET_RESCUED
/// @PARAM String The message when the unit dies. @DEFAULT $STR_TMF_TARGET_RESCUEDIED
/// @PARAM String The name of the task to appear in the briefing. This will automatically create a task and task waypoint. @DEFAULT Empty. Do not create a task
/// @PARAM String The description of the task in the briefing. @DEFAULT Empty. Do not add a task description
/// @PARAM String Init Line code of the object @DEFAULT "doStop this; removeAllWeapons this;"
/// @PARAM String A custom code string which will be executed when the target is free. I recommend against using this. @DEFAULT Empty
/// @PARAM String A custom code string which will be executed when the target dies. I recommend against using this. @DEFAULT Empty
/// @RETURN Array An array containing he variable which indicates the unit is freed, a variable indicating the unit is still alive and the task unique name. [SuccessVariable,AliveVariable,TaskUniqueName]
/// @REQUIRE MissionFramework
/// @REMARK The object itself is accessible through the ACV.
/// @EXAMPLE ["RU_Worker1", ACV_PosClick,"WEST", "You freed the hostage, congratulations!", "You did your best to save the hostage...","Hostage","Task Description"] call TYR_FNCT_Mission_Rescue
TYR_FNCT_Mission_Rescue = {
	// ["RU_Worker1",_grp,[6179.82,10399.1,0],"RESCUE",["You freed the hostage, congratulations!","You did your best to save the hostage...","jip_hostage_free","jip_hostage_alive"],"doStop this;"] call XfMissionCreateMissionUnit;
	private ["_class","_position","_object_name","_success_msg","_failure_msg","_taskname","_taskdesc","_jip_success_var","_jip_alive_var","_unit_init","_custom_code","_taskunique","_wp_pos","_init_line"];
	_class = _this select 0;
	_position = _this select 1;
	_jip_success_var = Format["JIPM_RESCUE_%1", (call TYR_FNCT_NextJipID)];
	_jip_alive_var = Format["JIPM_ALIVE_%1", (call TYR_FNCT_NextJipID)];
	_side = d_own_side;
	if (count _this > 2) then { _side = _this select 2;};
	_success_msg = localize "STR_TMF_TARGET_RESCUED";
	if (count _this > 3) then { _success_msg = _this select 3;};
	_failure_msg = localize "STR_TMF_TARGET_RESCUEDIED";
	if (count _this > 4) then { _failure_msg = _this select 4;};
	_taskname = "";
	if (count _this > 5) then { _taskname = _this select 5;};
	_taskdesc = "";
	if (count _this > 6) then { _taskdesc = _this select 6;};
	_init_line = "doStop this; removeAllWeapons this; ";
	if (count _this > 7) then { _init_line = _this select 7; };
	_custom_code = "";
	if (count _this > 8) then { _custom_code = _this select 8;};
	_custom_code_die = "";
	if (count _this > 9) then { _custom_code_die = _this select 9;};
	_taskunique = "";
	if (_taskName != "") then {
		if (_taskdesc == "") then {_taskdesc = format[localize "STR_TMF_TASKNODESC" ,_taskname];};
		_taskunique = Format["JIPM_TASK_%1", (call TYR_FNCT_NextJipID)];
		_wp_pos = _position;
		if (typeName (_position select 0) == "ARRAY") then { _wp_pos = (_position select 0); };
		_wp_pos = [_wp_pos, 100] call TYR_FNCT_RanCircle;
		[true,[Format["marker_%1",_taskunique], _wp_pos, "ICON", "ColorBlue", [0.7,0.7],_taskName,0,"hd_objective","",1]] call XfMissionMarker;
		[_taskunique, _taskName, _taskdesc, _taskName, _wp_pos, "Created",_jip_success_var,"",_jip_alive_var] call XfMissionTask;
	};
	["XfMissionCreateMissionUnit",[_class,_side,_position,"RESCUE",[_success_msg,_failure_msg,_jip_success_var,_jip_alive_var,_custom_code,_custom_code_die],_init_line]] call TYR_FNCT_ExecuteMissionCommandServer;
	[_jip_success_var,_jip_alive_var,_taskunique];
};

/// @FUNCTION
/// @NAME TYR_FNCT_Mission_Arrest
/// @DESC Creates a unit to arrest (the unit remains in place).
/// @ACCESS Public
/// @MP True
/// @PARAM Class The class of the unit to create
/// @PARAM Position Position of the unit, it can be given with a direction. In such a case add the direction to the position like this: [[0,0,0],0]
/// @PARAM String The side of the unit to arrest. @DEFAULT Enemy Side
/// @PARAM String The message when the unit is arrested. @DEFAULT $STR_TMF_TARGET_ARRESTED
/// @PARAM String The message when the unit dies. @DEFAULT $STR_TMF_TARGET_ARRESTDIED
/// @PARAM String The name of the task to appear in the briefing. This will automatically create a task and task waypoint. @DEFAULT Empty. Do not create a task
/// @PARAM String The description of the task in the briefing. @DEFAULT Empty. Do not add a task description
/// @PARAM String Init Line code of the object @DEFAULT ""
/// @PARAM String A custom code string which will be executed when the target is arrested. I recommend against using this. @DEFAULT Empty
/// @PARAM String A custom code string which will be executed when the target dies. I recommend against using this. @DEFAULT Empty
/// @RETURN Array An array containing he variable which indicates the unit is arrested, a variable indicating the unit is still alive and the task unique name. [SuccessVariable,AliveVariable,TaskUniqueName]
/// @REQUIRE MissionFramework
/// @REMARK The object itself is accessible through the ACV.
/// @EXAMPLE ["RU_Soldier_Pilot", ACV_PosClick,"EAST", "You caught the pilot, congratulations!", "Damn, you killed the pilot....","The Pilot","Task Description"] call TYR_FNCT_Mission_Rescue
TYR_FNCT_Mission_Arrest = {
	// ["RU_Soldier_Pilot",_grp,[5995.68,10323.6,0],"ARREST",["You caught the pilot, congratulations!","Damn, you killed the pilot....","jip_pilot_arrested","jip_pilot_alive"],"doStop this;"] call XfMissionCreateMissionUnit;
	private ["_class","_position","_object_name","_success_msg","_failure_msg","_taskname","_taskdesc","_jip_success_var","_jip_alive_var","_unit_init","_custom_code","_taskunique","_wp_pos","_init_line"];
	_class = _this select 0;
	_position = _this select 1;
	_jip_success_var = Format["JIPM_ARREST_%1", (call TYR_FNCT_NextJipID)];
	_jip_alive_var = Format["JIPM_ALIVE_%1", (call TYR_FNCT_NextJipID)];
	_side = d_own_side;
	if (count _this > 2) then { _side = _this select 2;};
	_success_msg = localize "STR_TMF_TARGET_ARRESTED";
	if (count _this > 3) then { _success_msg = _this select 3;};
	_failure_msg = localize "STR_TMF_TARGET_ARRESTDIED";
	if (count _this > 4) then { _failure_msg = _this select 4;};
	_taskname = "";
	if (count _this > 5) then { _taskname = _this select 5;};
	_taskdesc = "";
	if (count _this > 6) then { _taskdesc = _this select 6;};
	_init_line = "";
	if (count _this > 7) then { _init_line = _this select 7; };
	_custom_code = "";
	if (count _this > 8) then { _custom_code = _this select 8;};
	_custom_code_die = "";
	if (count _this > 9) then { _custom_code_die = _this select 9;};
	_taskunique = "";
	if (_taskName != "") then {
		if (_taskdesc == "") then {_taskdesc = format[localize "STR_TMF_TASKNODESC" ,_taskname];};
		_taskunique = Format["JIPM_TASK_%1", (call TYR_FNCT_NextJipID)];
		_wp_pos = _position;
		if (typeName (_position select 0) == "ARRAY") then { _wp_pos = (_position select 0); };
		_wp_pos = [_wp_pos, 100] call TYR_FNCT_RanCircle;
		[true,[Format["marker_%1",_taskunique], _wp_pos, "ICON", "ColorBlue", [0.7,0.7],_taskName,0,"hd_objective","",1]] call XfMissionMarker;
		[_taskunique, _taskName, _taskdesc, _taskName, _wp_pos, "Created",_jip_success_var,"",_jip_alive_var] call XfMissionTask;
	};
	["XfMissionCreateMissionUnit",[_class,_side,_position,"ARREST",[_success_msg,_failure_msg,_jip_success_var,_jip_alive_var,_custom_code,_custom_code_die],_init_line]] call TYR_FNCT_ExecuteMissionCommandServer;
	[_jip_success_var,_jip_alive_var,_taskunique];
};


/// @FUNCTION
/// @NAME TYR_FNCT_Mission_Lead
/// @DESC 
/// @ACCESS Public
/// @MP True
/// @PARAM Class The class of the unit to create
/// @PARAM Position Position of the unit, it can be given with a direction. In such a case add the direction to the position like this: [[0,0,0],0]
/// @PARAM Position Destination of the unit.
/// @PARAM String The side of the unit to lead. @DEFAULT Own Side
/// @PARAM String The message when the unit is at destination. @DEFAULT $STR_TMF_TARGET_LEAD
/// @PARAM String The message when the unit dies. @DEFAULT $STR_TMF_TARGET_DIED
/// @PARAM String The name of the task to appear in the briefing. This will automatically create a task and task waypoint. @DEFAULT Empty. Do not create a task
/// @PARAM String The description of the task in the briefing. @DEFAULT Empty. Do not add a task description
/// @PARAM String Init Line code of the object @DEFAULT "doStop this; removeAllWeapons this;"
/// @PARAM String A custom code string which will be executed when the target is at destination. I recommend against using this. @DEFAULT Empty
/// @RETURN Array An array containing he variable which indicates the unit is at destination, a variable indicating the unit is still alive and the task unique name. [SuccessVariable,AliveVariable,TaskUniqueName]
/// @REQUIRE MissionFramework
/// @REMARK The object itself is accessible through the ACV.
/// @EXAMPLE ["USMC_Soldier_Officer", ACV_PosClick, getPos Player,"WEST", "You brought him back", "The officer died!","The lost officer","Task Description"] call TYR_FNCT_Mission_Lead
TYR_FNCT_Mission_Lead = {
	// ["USMC_Soldier_Officer",_grp,[5933.32,10400.7,0],"LEAD",["You brought our soldier back, congratulations!","You did your best...","jip_lead_soldier","jip_soldier_alive",[[5895.41,9964.44,0],[20,20],0,false]],"doStop this; removeAllWeapons this;"] call XfMissionCreateMissionUnit;
	private ["_class","_position","_object_name","_success_msg","_failure_msg","_taskname","_taskdesc","_jip_success_var","_jip_alive_var","_unit_init","_custom_code","_taskunique","_wp_pos","_init_line"];
	_class = _this select 0;
	_position = _this select 1;
	_destination = _this select 2;
	_jip_success_var = Format["JIPM_LEAD_%1", (call TYR_FNCT_NextJipID)];
	_jip_alive_var = Format["JIPM_ALIVE_%1", (call TYR_FNCT_NextJipID)];
	_side = d_own_side;
	if (count _this > 3) then { _side = _this select 3;};
	_success_msg = localize "STR_TMF_TARGET_LEAD";
	if (count _this > 4) then { _success_msg = _this select 4;};
	_failure_msg = localize "STR_TMF_TARGET_DIED";
	if (count _this > 5) then { _failure_msg = _this select 5;};
	_taskname = "";
	if (count _this > 6) then { _taskname = _this select 6;};
	_taskdesc = "";
	if (count _this > 7) then { _taskdesc = _this select 7;};
	_init_line = "doStop this; removeAllWeapons this; ";
	if (count _this > 8) then { _init_line = _this select 8; };
	_custom_code = "";
	if (count _this > 9) then { _custom_code = _this select 9;};
	_custom_code_die = "";
	if (count _this > 10) then { _custom_code_die = _this select 10;};
	_taskunique = "";
	if (_taskName != "") then {
		if (_taskdesc == "") then {_taskdesc = format[localize "STR_TMF_TASKNODESC" ,_taskname];};
		_taskunique = Format["JIPM_TASK_%1", (call TYR_FNCT_NextJipID)];
		_wp_pos = _position;
		if (typeName (_position select 0) == "ARRAY") then { _wp_pos = (_position select 0); };
		_wp_pos = [_wp_pos, 100] call TYR_FNCT_RanCircle;
		[true,[Format["marker_%1",_taskunique], _wp_pos, "ICON", "ColorBlue", [0.7,0.7],_taskName,0,"hd_objective","",1]] call XfMissionMarker;
		[_taskunique, _taskName, _taskdesc, _taskName, _wp_pos, "Created",_jip_success_var,"",_jip_alive_var] call XfMissionTask;
	};
	// Create the destination marker
	_wp_pos = _destination;
	if (typeName (_destination select 0) == "ARRAY") then { _wp_pos = (_destination select 0); };
	[false,[Format["marker_%1_dest",_taskunique], _wp_pos, "ICON", "ColorBlue", [0.7,0.7],_taskName,0,"hd_end","Destination",1]] call XfMissionMarker;
	
	// Create the unit to lead
	["XfMissionCreateMissionUnit",[_class,_side,_position,"LEAD",[_success_msg,_failure_msg,_jip_success_var,_jip_alive_var,[_destination,[20,20],0,false],_custom_code,_custom_code_die],_init_line]] call TYR_FNCT_ExecuteMissionCommandServer;
	[_jip_success_var,_jip_alive_var,_taskunique];
};


/// @FUNCTION
/// @NAME TYR_FNCT_Mission_Conquer
/// @DESC 
/// @ACCESS Public
/// @MP True
/// @PARAM Position The center of the zone to conquer, it can be given with a radius. In such a case add the radius to the position like this: [[0,0,0],0] The Default Radius is 300.
/// @PARAM String Side of the enemy which need to be expelled @DEFAULT d_enemy_side
/// @PARAM String Success message when the objective is conquered @DEFAULT $STR_TMF_TARGET_CONQUERED
/// @PARAM String The name of the task to appear in the briefing. This will automatically create a task and task waypoint. @DEFAULT Empty. Do not create a task
/// @PARAM String The description of the task in the briefing. @DEFAULT Empty. Do not add a task description
/// @PARAM String A custom code string which will be executed when the zone is conquered. I recommend against using this. @DEFAULT Empty
/// @RETURN Array An array containing only the variable which indicates the success of the objective and the task unique name. [SuccessVariable,TaskUniqueName]
/// @REQUIRE MissionFramework
/// @REMARK
/// @EXAMPLE [ACV_PosClick, "EAST", "The town is freed"] call TYR_FNCT_Mission_Conquer
TYR_FNCT_Mission_Conquer = {
	// ["Land_HelipadEmpty_F",_town_Lopatino_pos,"CONQUER",["The town of Lopatino is now clear!",[_town_Lopatino_pos, [(_town_Lopatino_size * 1.5),(_town_Lopatino_size * 1.5)], 0, false],"jip_lopatino_freed"]] call XfMissionCreateMissionObject;
	private ["_center","_radius","_success_msg","_conquer_side","_taskname","_taskdesc","_jip_var","_action_title","_custom_code","_taskunique","_wp_pos","_init_line"];
	_center = _this select 0;
	_radius = 300;
	if (typeName (_center select 0) == "ARRAY") then {
		_radius = (_center select 1);
		_center = (_center select 0);
	};
	_jip_var = Format["JIPM_CONQUER_%1", (call TYR_FNCT_NextJipID)];
	_conquer_side = d_enemy_side;
	if (count _this > 1) then { _conquer_side = _this select 1;};
	_success_msg = localize "STR_TMF_TARGET_CONQUERED";
	if (count _this > 2) then { _success_msg = _this select 2;};
	_taskname = "";
	if (count _this > 3) then { _taskname = _this select 3;};
	_taskdesc = "";
	if (count _this > 4) then { _taskdesc = _this select 4;};
	_custom_code = "";
	if (count _this > 5) then { _custom_code = _this select 5;};
	_taskunique = "";
	if (_taskName != "") then {
		if (_taskdesc == "") then {_taskdesc = format[localize "STR_TMF_TASKNODESC" ,_taskname];};
		_taskunique = Format["JIPM_TASK_%1", (call TYR_FNCT_NextJipID)];
		_wp_pos = _center;
		if (typeName (_center select 0) == "ARRAY") then { _wp_pos = (_center select 0); };
		_wp_pos = [_wp_pos, 100] call TYR_FNCT_RanCircle;
		[true,[Format["marker_%1",_taskunique], _wp_pos, "ICON", "ColorBlue", [0.7,0.7],_taskName,0,"hd_objective","",1]] call XfMissionMarker;
		[_taskunique, _taskName, _taskdesc, _taskName, _wp_pos, "Created",_jip_var,""] call XfMissionTask;
	};
	["XfMissionCreateMissionObject",["Land_HelipadEmpty_F",_center,"CONQUER",[_success_msg,[_center, [_radius,_radius], 0, false],_conquer_side,_jip_var,_custom_code]]] call TYR_FNCT_ExecuteMissionCommandServer;
	[_jip_var,_taskunique];
};

/// @FUNCTION
/// @NAME TYR_FNCT_Mission_Civilians
/// @DESC 
/// @ACCESS Public
/// @MP True
/// @PARAM Position The center of the zone to conquer, it can be given with a radius. In such a case add the radius to the position like this: [[0,0,0],0] The Default Radius is 300.
/// @PARAM String The side of the civilians
/// @PARAM Scalar The maximum number of civilians. It will always be a random value between this number and it's half.
/// @RETURN Nothing
/// @REQUIRE MissionFramework
/// @REMARK The group with all civilians is accessible through the ACV.
/// @EXAMPLE [ACV_PosClick, "EAST", 20] call TYR_FNCT_Mission_Civilians;
TYR_FNCT_Mission_Civilians = {
	//[[d_town_pos,200],"EAST",10] call XfMissionCivilianSpawn
	private ["_side","_position","_amount"];
	_position = _this select 0;
	_side = _this select 1;
	if (typename _side == "STRING") then {
		_side = toUpper(_side);
	} else {
		_side = d_enemy_side;
	};
	_amount = _this select 2;
	if (typeName _amount != "SCALAR") then { _amount = 20; };
	["XfMissionCivilianSpawn",[_position,_side,_amount]] call TYR_FNCT_ExecuteMissionCommandServer;
};

TYR_FNCT_Compiled = true;