// 2014-02-01 Tyrghen
//---------------------------------------------
// Usage:
//  [object,"STR_ACTION_TITLE","my_jip_variable",true,true] call edt_fnc_setAction;
//  [object,"STR_ACTION_TITLE","my_jip_variable"] call edt_fnc_setAction;
//---------------------------------------------
// Purpose:
// Adds an action to an object, that action will, when used, will set a JIP synchronised variable to the given value and will optionally hide the action.
// 
// 
// Parameters:
// - The object to add the action to
// - The title of the action displayed to the player (can be a magic string starting with STR_, it will be translated automatically)
// - The name of the JIP synchronised variable set by the action. You can retrieve the content of this variable with EDT_fnc_getVar
// Optional Parameters:
// - The value given to the variable (by default: true)
// - A boolean indicating whether the action should be hidden after the variable has been set
//
// Returns:
// The action ID
//---------------------------------------------
// Objectif:
// Ajoute une action a un objet, cette action, une fois utilise, va assigner une valeur donne  la variable synchronise en JIP et va ventuellement masquer l'action.
//
// Paramtres:
// - L'objet auquel on souhaite ajouter l'action
// - Le titre de l'action montr au joueur (peut tre une chane magique commenant par STR_, elle sera traduite automatiquement)
// - Le nom d'une variable synchronise en JIP qui sera assigne par l'action. Vous pouvez rcuprer le contenu de cette variable avec EDT_fnc_getVar
// Paramtres optionnels
// - La valeur donne  la variable (par dfaut: true)
// - Un boolen indiquant si l'action doit tre masque aprs que la variable est assigne.
//
// Renvoie:
// L'ID de l'action
//---------------------------------------------

//#define __FORCE_TRACE__
//#define __FORCE_DEBUG__
//#define __FORCE_INFO__
#define THIS_FILE "editor\fn_setAction.sqf"
#include <tmf_constants.h>
#include <tmf_macros.h>

TRACE_1("Start",_this);
TMFPROFILERSTART;

private ["_object","_action_title","_variable","_value","_arguments","_condition","_hideonuse","_action_id"];
DEFAULT_PARAM(_object,0,objNull);
DEFAULT_PARAM(_action_title,1,"");
DEFAULT_PARAM(_variable,2,"");
DEFAULT_PARAM(_value,3,true);
DEFAULT_PARAM(_hideonuse,4,false);
DEFAULT_PARAM(_check_alive,5,true);

_action_title = (_action_title) call FFUNC(translate);
_condition = "";
if (_check_alive) then {
	_condition = "(alive _target)";
};
if (_hideonuse) then {
	if (_check_alive) then {
		_condition = _condition + " && ";
	};
	if (IS_SCALAR(_value) || IS_STRING(_value)) then {
		_condition = Format["%1 (vehicle player == player) && {((['%2','%3'] call %4) != '%3')}",_condition,_variable,_value,QUOTE(EFUNC(getVar))];
	};
	if (IS_BOOL(_value)) then {
		if (_value) then {
			_condition = Format["%1 (vehicle player == player) && {!(['%2',%3] call %4)}",_condition,_variable,(!_value),QUOTE(EFUNC(getVar))];
		} else {
			_condition = Format["%1 (vehicle player == player) && {(['%2',%3] call %4)}",_condition,_variable,(!_value),QUOTE(EFUNC(getVar))];
		};
	};
};
_arguments = [];
SETVAL(_arguments,0,ARR_2(_variable,_value));
SETVAL(_arguments,1,EFUNC(setVar));

TRACE_1("Action",ARR_3(_action_title,_condition,_arguments));
_action_id = _object addAction [_action_title, QPATH(addActionLocal.sqf),_arguments,0,True,True,"",_condition];

TRACE_1("End",_action_id);
TMFPROFILERSTOP;
_action_id;